# -*- coding: utf-8 -*-

from django import http
from django.views.generic import View
from django.core.urlresolvers import reverse
from django.utils.functional import Promise
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import force_unicode
from django.template.loader import render_to_string
from django.template import RequestContext
from django.shortcuts import get_object_or_404, render_to_response

from .settings import *
from .menu import MenuActives
from .utils import LazyEncoder

import json
import copy
import datetime


class MenuMixin(object):
    def __init__(self, *args, **kwargs):
        class_menu = getattr(self, SV_CONTEXT_VARNAME, [])
        kwargs_menu = kwargs.pop(SV_CONTEXT_VARNAME, class_menu)
        setattr(self, SV_CONTEXT_VARNAME, MenuActives(kwargs_menu))
        super(MenuMixin, self).__init__(*args, **kwargs)

    def get_context(self):
        context = super(MenuMixin, self).get_context()
        context.update({SV_CONTEXT_VARNAME: getattr(self, SV_CONTEXT_VARNAME)})
        return context


class JSONMixin(object):
    def render_json(self, context={}, ok=True):
        """
        Serialize the context variable to json. Aditionally add ``success``
        attribute with `True` value by default. This can changed with `ok`
        parameter.
        """

        response = {'success': ok}
        response.update(context)
        return self._render_json(response)

    def _render_json(self, context, noformat=True):
        """
        Returns a JSON response containing 'context' as payload
        """
        if not noformat:
            return http.HttpResponse(json.dumps(context, indent=4, \
                cls=LazyEncoder, sort_keys=True), mimetype="text/plain")
        return http.HttpResponse(json.dumps(context, cls=LazyEncoder), \
                                                    mimetype="text/plain")

    def render_json_error(self, errors_data, aditional=[], context={}, form=None):
        """
        Helper method for serialize django form erros in a estructured and friendly
        json for javascript form validators.

        How to use this::
            
            class MyView(SuperView):
                def post(request):
                    form = MyForm()
                    if form.is_valid():
                        return self.render_json()

                    return self.render_json_error(form.errors)
        """

        assert isinstance(aditional, (list, tuple)), "aditional parameter must be a list or a tuple."
        assert isinstance(context, dict), "context parameter must be a dict"

        response_dict = {'success': False, 'errors': {'global':[], 'form':{}, 'fields':{}}}

        if isinstance(errors_data, (unicode, str, Promise)):
            response_dict['errors']['global'].extend([errors_data])

        elif isinstance(errors_data, (list, tuple)):
            response_dict['errors']['global'].extend(list(errors_data))

        elif isinstance(errors_data, dict):
            errors = copy.deepcopy(errors_data)

            if "__all__" in errors:
                non_field_errors = list(errors.pop('__all__'))
                response_dict['errors']['global'].extend(non_field_errors)

            response_dict['errors']['form'].update(errors)

            if form:
                for field in response_dict['errors']['form'].keys():
                    response_dict['errors']['fields'][field] = {'name': form[field].label}
                
        if aditional:
            response_dict['errors']['global'].extend(aditional)

        if context:
            response_dict.update(context)

        return self._render_json(response_dict)


class ResponseMixin(object):
    template_path = None

    def get_context(self):
        return {'view': self}

    def render_to_response(self, template=None, context={}, **kwargs):  
        template = template if template else self.template_path
        context = context if context else {}

        response_context = {}
        response_context.update(context)
        response_context.update(self.get_context())

        return render_to_response(template, response_context,
            context_instance=RequestContext(self.request), **kwargs)

    def render_redirect(self, url):
        return http.HttpResponseRedirect(url)


class SuperView(JSONMixin, MenuMixin, ResponseMixin, View):
    pass



# -*- coding: utf-8 -*-

from django.core.serializers.json import DjangoJSONEncoder
from django.utils.functional import Promise
from django.utils.encoding import force_unicode

try:
    from django.utils.timezone import localtime
except ImportError:
    localtime = lambda x: x

import datetime

class LazyEncoder(DjangoJSONEncoder):
    """
    JSON encoder class for encode correctly traduction strings.
    Is for ajax response encode.
    """

    def default(self, obj):
        if isinstance(obj, Promise):
            return force_unicode(obj)
        elif isinstance(obj, datetime.datetime):
            obj = localtime(obj)
        return super(LazyEncoder, self).default(obj)




# -*- coding: utf-8 -*-

"""
Stream decoarators. 

Allows to a class view method (get/post/head/delete) return 
generator object instad of HttpResponse instance. Useful if you 
want to return large amounts of data, and not use much memory.

Example::
    
    from django.views.generic import View
    import io

    class SomeView(View):

        @stream_view(content_type="application/xml")
        def get(self, request):
            with io.open("somelargefile.xml","r") as f:
                for line in f:
                    yield line

"""

from django.http import HttpResponse
from django.core.exceptions import ImproperlyConfigured

import types
import functools


VALID_STREAM_RESPONSES = (
    types.FunctionType,
    types.LambdaType,
    types.GeneratorType,
)

def stream_view(content_type=None):
    if content_type is None:
        raise ImproperlyConfigured("content_type parameter can not be None")

    def _decorator(func):
        @functools.wraps(func)
        def _wrapper(*args, **kwargs):
            response = func(*args, **kwargs)
            if isinstance(response, VALID_STREAM_RESPONSES):
                return HttpResponse(response, content_type=content_type)
            return response

        return _wrapper
    return _decorator



# -*- coding: utf-8 -*-

from .settings import *

import re

attr_rx1 = re.compile(r"^in_(.+)$", flags=re.U)
attr_rx2 = re.compile(r"^if_menu(\d+)_is_(.+)$", flags=re.U)

class MenuActives(object):
    """
    Menu context object.
    """

    def __init__(self, menu_actives):
        if isinstance(menu_actives, (list, tuple)):
            self.menu_actives = menu_actives
        else:
            self.menu_actives = (menu_actives,)

    def __getattr__(self, attr):
        res1 = attr_rx1.search(attr)
        res2 = attr_rx2.search(attr)

        if res1:
            def default_method(*args):
                if res1.group(1) in self.menu_actives:
                    return True if not SV_CSS_MENU_ACTIVE else SV_CSS_MENU_ACTIVE
                else:
                    return False if not SV_CSS_MENU_ACTIVE else ''
            return default_method

        elif res2:
            menu_index, menu_name = res2.groups()
            def default_method(*args):
                try:
                    if self.menu_actives[int(menu_index)] == menu_name:
                        return True if not SV_CSS_MENU_ACTIVE else SV_CSS_MENU_ACTIVE
                    else:
                        return False if not SV_CSS_MENU_ACTIVE else ''
                except IndexError:
                    return False if not SV_CSS_MENU_ACTIVE else ''
            return default_method

        return super(MenuActives, self).__getattr__(attr)



# -*- coding: utf-8 -*-

from django import template
from django.utils.safestring import mark_safe

register = template.Library()

@register.simple_tag(takes_context=True, name='call_view_method')
def call_view_method(context, method_name, *args, **kwargs):
    view = context['view']
    default_method = lambda x, *args, **kwargs: ''

    view.current_context = context
    method_response = getattr(view, method_name, default_method)(*args, **kwargs)
    del view.current_context

    return method_response



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import superview

setup(
    name = 'django-superview',
    version = ":versiontools:superview:",
    description = "Generic View class with extra funionality for Django",
    long_description = "",
    keywords = 'django, view',
    author = 'Jesús Espino García & Andrei Antoukh',
    author_email = 'jespinog@gmail.com, niwi@niwi.be',
    url = 'https://github.com/jespino/django-superview',
    license = 'BSD',
    include_package_data = True,
    packages = find_packages(),
    install_requires=[
        'distribute',
    ],
    setup_requires = [
        'versiontools >= 1.8',
    ],
    classifiers = [
        "Programming Language :: Python",
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ]
)
